/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: collide.h
*/

#include "core.h"

int sign ( double  n )
{
    if (n < 0) return (-1);

    if (n > 0) return (1);

    return(0);
}



/* Returns true if the point is in the rectangle */
// defined in order ABDC
bool point_in_rect(vector a, vector b, vector c, vector d, vector p)
{
    vector AB_norm = (d - a).norm();    // Vector normal to AB
    vector BC_norm = (a - b).norm();    // Vector normal to AB
    vector CD_norm = (b - c).norm();    // Vector normal to AB
    vector DA_norm = (c - d).norm();    // Vector normal to AB
    
    if (sign(dot(p-a, AB_norm)) != sign(dot(c-a, AB_norm))) return(false);

    if (sign(dot(p-b, BC_norm)) != sign(dot(d-b, BC_norm))) return(false);
         
    if (sign(dot(p-c, CD_norm)) != sign(dot(a-c, CD_norm))) return(false);

    if (sign(dot(p-d, DA_norm)) != sign(dot(b-d, DA_norm))) return(false);
     
    return(true);
}


bool point_on_tri(vector a, vector b, vector c, vector p)
{
    
    vector AB_norm = (c - a).norm();    // Vector normal to AB
    vector BC_norm = (a - b).norm();    // Vector normal to BC
    vector CA_norm = (b - c).norm();    // Vector normal to CA
    
     if (sign(dot(p-a, AB_norm)) != sign(dot(c-a, AB_norm))) return(false);

     if (sign(dot(p-b, BC_norm)) != sign(dot(a-b, BC_norm))) return(false);
         
     if (sign(dot(p-c, CA_norm)) != sign(dot(b-c, CA_norm))) return(false);

     
     return(true);
}

bool check_point_tri(vector a, vector b, vector c, vector pos, vector dir, double trigger, vector &i_dir)
{

    i_dir = zero;
           
           
    vector u1 = cross(pos - a, pos - b);
    vector u2 = cross(pos - b, pos - c);
    vector u3 = cross(pos - c, pos - a);
   
   
//    if (dot(u1, dir) > 0)  return(false);
//    if (dot(u2, dir) > 0)  return(false);
//    if (dot(u3, dir) > 0)  return(false);

    double d1, d2, d3;
    
    d1 = dot(u1, dir);
    d2 = dot(u2, dir);
    d3 = dot(u3, dir);
    
    bool in = 0;
    
    if (d1 > 0 && d2 > 0 && d3 > 0) in = 1;
//    /if (d1 < 0 && d2 < 0 && d3 < 0) in = 1;
    
    if (in == 0) return (false);




    vector n = cross(b-a, c-a);  // get normal of rectangle
    
    n.normalize();


    dir.normalize();
    
    
    vector p1 = pos;
    vector p2 = pos + dir;
           
           
           
    double denom = dot(n, p2- p1); // N dot Dir
   
    if (fabs(denom) < epsilon) 
    {
        write_console("denom too small\n");
        return (false);        
    }
    
    // Solve for t of parametric line equation
    double t = dot(n, a - p1) / denom;

    char tmp[400];


//    double  trigger= 4;


    // In front of plane?
    if (t >  trigger)  return (false);

    // Find point on plane
  //  vector p = p1 + t * (p2 - p1);
    
    // Make sure point is in the triangle
//    if(!point_on_tri(a, b, c, p)) return (false);

    
    i_dir = (trigger - t) * dir;

  //  sprintf(tmp, "Impact: (%8.2f, %8.2f, %8.2f) t: %f \n", i_dir.x, i_dir.y, i_dir.z, t);

  //  write_console(tmp);

    return true;    
}






/* Check collisions between two objects */
void world_class::check_object_collide( dobject_class *obj1, dobject_class *obj2 )          
{                       
    // Do not compare object with itself                 
    if (obj1 == obj2) return;    
char tmp[400];




    double r1 = obj1->size.mag() / 2.0;
    double r2 = obj2->size.mag() / 2.0;
                
    double obj_dist = (obj2->cur_state.pos - obj1->cur_state.pos).mag();                       


    sprintf(tmp, "Obj1: %x Obj2: %x R2 %f  r2: %f   dist: %f ", obj1, obj2, r1, r2, obj_dist);

    //write_console(tmp);
debug(tmp);




    if (obj_dist < r1 + r2)
    {

debug("dist check");
        // need to cross check point in both objects
                         
        double trigger_dist = (obj2->mass_radius + obj1->mass_radius) / 2.0;
                        
                                       
        vector dir;

     
     
        for (int t = 0; t < obj2->num_face_points/3; t++)
        {
            // check points in obj1 agasint faces in obj2
            for (int i = 0; i < obj1->total_points; i++)                                       
            {                                           


                vector p_vel = obj1->points[i].cur_state.vel;
            
                quaternion_rotate_vector(obj1->orientation, p_vel);    
          
                p_vel += obj1->cur_state.vel;

           //     p_vel.normalize();
          


                if( check_point_tri(obj2->face_masses[t*3].mass->world_pos, 
                                    obj2->face_masses[t*3+1].mass->world_pos,
                                    obj2->face_masses[t*3+2].mass->world_pos,
                                    obj1->points[i].world_pos, p_vel, 0.2, dir))
                {     
   
                    //dir.normalize();
  
//                    obj1->points[i].ext_force = -object_force * dir;                    

                    double f = 4.0;// + p_vel.mag();

                // rotate force vector back into world space
               // quaternion_rotate_vector(obj1->orientation.conjugate(), dir);    

//char tmp[400];
    sprintf(tmp, "Obj1: %x Obj2: %x Impact: (%8.2f, %8.2f, %8.2f)", obj1, obj2, dir.x, dir.y, dir.z);

  //  write_console(tmp);
//debug(tmp);

                    obj1->points[i].ext_force += (f) * dir  ;                    

              //      obj2->face_masses[t*3].mass->ext_force -= f* dir;                    
//                    obj2->face_masses[t*3+1].mass->ext_force -= f * dir;                    
//                    obj2->face_masses[t*3+2].mass->ext_force -= f * dir;                    

                    
                    // Apply force to face
//                    obj2->face_masses[t*3].mass->ext_force = -object_force * dir;                  
//                    obj2->face_masses[t*3+1].mass->ext_force = -object_force * dir;                  
//                    obj2->face_masses[t*3+2].mass->ext_force = -object_force * dir;                  
                                   
                }    
                
                
            } 
        }  
            

/*
        for (int i = 0; i < obj1->total_points; i++)                                       
            for (int j = 0; j < obj2->total_points; j++)
            {
                vector delta = obj2->points[j].world_pos - obj1->points[i].world_pos;
                                                                    
                double dist = delta.mag();
                           
                if (dist < epsilon) dist = epsilon;                            
                           
                vector imp_dir = delta / dist;

                // Apply foces to both objects if in interference zone
                if (dist < trigger_dist)
                {
                    vector force =  object_force * imp_dir * (trigger_dist - dist)*(trigger_dist - dist);

                    obj2->points[j].ext_force += force;
                    obj1->points[i].ext_force -= force;
                }
            }
        
     */             
    }
}
/* End of check_object_collide */


/* Check world collisions */
void world_class::check_collide( void )
{

    dobject_class *d = objects;

    vector probe (probe_pos.x, probe_pos.y, probe_pos.z);
    vector probe_dir (camera_dir.x, camera_dir.y, camera_dir.z);
    
    vector probe_vel (camera_vel.x, camera_vel.y, camera_vel.z);
   
    state probe_s(probe_dir, probe_vel);
   
   
    // Zero all external forces
    while (d)
    {
        for (unsigned int p = 0; p < d->total_points; p++)
            d->points[p].ext_force = zero;
   
        d->apply_ground_friction = false;
   
        d=d->next;
    }   
    
    d = objects;    

    // Check all obejects
    while (d)
    {
       
        vector dir;


        
        
        double r1 = d->size.mag() / 2.0;
                
        double obj_dist = (probe - d->cur_state.pos).mag();                       

        // only check object if it's in range of the probe
     //   if (obj_dist < r1)        
        {
            // check camera probe against face triangles
            for (int t = 0; t < d->num_face_points/3; t++)
            {
                if( check_point_tri(d->face_masses[t*3].mass->world_pos, 
                                     d->face_masses[t*3+1].mass->world_pos,
                                     d->face_masses[t*3+2].mass->world_pos,
                                     probe, probe_dir, 4, dir))
                {
                    dir.normalize();
                    
                    
                  //  camera_force = 10000;
                    
                    // Apply force to face
                    d->face_masses[t*3].mass->ext_force += camera_force * dir;                  
                    d->face_masses[t*3+1].mass->ext_force += camera_force * dir;                  
                    d->face_masses[t*3+2].mass->ext_force += camera_force * dir;                  
                                    
                    // Slow down camera
                    camera_vel.x *= 0.75;              
                    camera_vel.y *= 0.75;               
                    camera_vel.z *= 0.75;           
                }
            
            }
        }
        
        
        
        
        
      
      //  vector state_rect_intersection(vector a, vector b, vector c, vector d, state s)
        
//    vector dir;

//    if (state_rect_intersection( v[0], v[1], v[2], v[3], probe, probe_dir, dir))
//    {

//        camera_vel.x = 0; /// v1.x;
//        camera_vel.y = 0;//v1.y;
//        camera_vel.z = 0;// =0;// v1.z;
//    }
         // Mass inside of probe ? 
//            if (dist < probe_radius)
//            {
//                if (dist < epsilon) dist = epsilon;

//                if (dist < epsilon) dist = epsilon;
//                    d->points[p].ext_force += camera_force * probe_dir / dist;
//            }              
    
        
        
        
        
        
        
        
        
        
        
        
        
        
        // check against camera
     /*   for (unsigned int p = 0; p < d->total_points; p++)
        {
            double dist = (probe - d->points[p].world_pos).mag();
                                                                               
            // Mass inside of probe ? 
            if (dist < probe_radius)
            {
                if (dist < epsilon) dist = epsilon;

                if (dist < epsilon) dist = epsilon;
                    d->points[p].ext_force += camera_force * probe_dir / dist;
            }                  
        }*/


        // Check against floor
        for (unsigned int p = 0; p < d->total_points; p++)
        {
           if (d->points[p].world_pos.y < 0)
           {
                d->points[p].ext_force.y -= terrain_force * d->points[p].world_pos.y;

                d->apply_ground_friction = true;
            }
        }



    

        // Check against other objects
        dobject_class *d1 = objects;

        while (d1)
        {


            check_object_collide(d, d1);

            d1=d1->next;            
        }

        d=d->next;        
    }
 
 
    // Rotate and translate all forces into object space 
 
    d = objects;

    while (d)
    {
        for (int i = 0; i < d->total_points; i++)
            quaternion_rotate_vector(d->orientation.conjugate(), d->points[i].ext_force);
        
        d = d->next;                
    }
 
}
/* End of world_class::check_collide */











