/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: graphics.cpp
*/

#include "core.h"

/* Update camera angle and position */
void world_class::update_camera( void )
{
    POINT p;

    // Get mouse
    GetCursorPos(&p);
   
    // reset to base
    SetCursorPos(200, 200);

    // Calculate mouse movement    
    camera_ang.x -= (p.y - 200) / 400.0;
    camera_ang.y += (p.x - 200) / 400.0;

    // Range check x angle
    if (camera_ang.x > M_PI / 2) camera_ang.x = M_PI / 2;
    if (camera_ang.x < -M_PI / 2) camera_ang.x= -M_PI / 2;
    
    // Set camera direction
    camera_dir.x = sin(camera_ang.y);
    camera_dir.z = cos(camera_ang.y);    
    camera_dir.y = sin(camera_ang.x);    
               
    if(GetAsyncKeyState(0x57)) // W key Forward
        camera_vel += cam_speed * camera_dir;
    
    if(GetAsyncKeyState(0x53)) // S Backward
        camera_vel -= cam_speed * camera_dir;

    if(GetAsyncKeyState(0x41)) // A - Strafe left
    {
        // Add a vector that's 90 degrees out of phase with the direction
        camera_vel.x -= 0.7 * cam_speed * camera_dir.z;
        camera_vel.z += 0.7 * cam_speed * camera_dir.x;
    }

    if(GetAsyncKeyState(0x44)) // D - Strafe right
    {
        // Add a vector that's 90 degrees out of phase with the direction
        camera_vel.x += 0.7 * cam_speed * camera_dir.z;
        camera_vel.z -= 0.7 * cam_speed * camera_dir.x;
    }

    camera_vel *= 0.90; // Reduce speed

    camera_pos += camera_vel;   // New camera position

    // Range check y position
    if (camera_pos.y < 1) camera_pos.y = 1;

    // Setup view vector
	D3DXVECTOR3 up(0.0f, 1.0f, 0.0f);
   
    D3DXVECTOR3  target = camera_pos + camera_dir;

	D3DXMatrixLookAtLH(&dx_view, &camera_pos, &target, &up);
	
	// Position the probe
	
    probe_pos = camera_pos + probe_dist * camera_dir;	
    

    // setup view frustrum bounds

//    camera_dir.x = cos(camera_ang);
//    camera_dir.y = sin(camera_ang);   

    camera_l_bound.x = cos(camera_ang.y+0.5);
    camera_l_bound.y = sin(camera_ang.y+0.5);

    camera_r_bound.x = cos(camera_ang.y-0.5);
    camera_r_bound.y = sin(camera_ang.y-0.5);
    
}
    


/* End of update camera */


   
/* Display text */
void world_class::display_text(int x, int y, char *text)
{
    // Define bounding rectangle
    RECT r;
    r.left=x;
    r.right=display_x;
    r.top=y;
    r.bottom=display_y;

    dx_font->DrawText(NULL, text, -1, &r, 0, dx_white );
}
/* End of display_text */


/* Display FPS data */
void world_class::showstat( void )
{
    char b[400];

    sprintf(b, "FPS: %4d Drawn Tri: %4d", fps, terrain.num_triangles);

    display_text(0,0,b);    
}
/* End of graphics_showfps */






/* Render the frame */
void world_class::render(void)
{
    // Check for wireframe mode
    if (wireframe)
      	dx_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);    
    else
      	dx_device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);    

			
    // Set projection transform
	dx_device->SetTransform(D3DTS_PROJECTION, &dx_projection);

    // Set view transform
    dx_device->SetTransform(D3DTS_VIEW, &dx_view);   

    // No lighting
	dx_device->SetRenderState(D3DRS_LIGHTING, FALSE);    

    // Erase background and reset zbuffer
	dx_device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x00000000, 1.0f, 0);

    // Start active render
    dx_device->BeginScene(); 

    // Set vertex format
    dx_device->SetFVF(dx_fvf); 

    // Draw terrain
    terrain.render();

    // Display upper stat bar
    showstat();

    // Done drawing
    dx_device->EndScene();

    // Display
    dx_device->Present(NULL, NULL, NULL, NULL);

    // Bump frame counter
    frames++;
}
/* End of graphics_render */



/* Start graphics system */
int world_class::start_graphics(HWND hwnd)
{
    if ((dx_object = Direct3DCreate9(D3D_SDK_VERSION)) == NULL) return(1);

    D3DPRESENT_PARAMETERS dx_param;

    ZeroMemory(&dx_param, sizeof(dx_param));

    dx_param.Windowed = !fullscreen;
    dx_param.SwapEffect = D3DSWAPEFFECT_DISCARD;
    dx_param.hDeviceWindow = hwnd;
    dx_param.BackBufferFormat = D3DFMT_A8R8G8B8; 
    dx_param.BackBufferWidth = display_x;
    dx_param.BackBufferHeight = display_y;

	dx_param.EnableAutoDepthStencil = true; 
    dx_param.AutoDepthStencilFormat = D3DFMT_D16;

    dx_object->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
//                            D3DCREATE_HARDWARE_VERTEXPROCESSING,
                            D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                            &dx_param, &dx_device);


    // Create tarrain floor
    //create_terrain();

    // Setup a basic font
    D3DXCreateFont(dx_device, 20, 0, FW_BOLD, 0, FALSE, 
                   DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, 
                   DEFAULT_PITCH | FF_DONTCARE, "Arial", &dx_font);

    // Load texture
    if (D3DXCreateTextureFromFile(dx_device,"rocks3.jpg", &dx_texture1) != D3D_OK)
    {
        debug("Unable to load texture");
        return(1);
    }

    if (D3DXCreateTextureFromFile(dx_device,"tx5.bmp", &dx_texture2) != D3D_OK)
    {
        debug("Unable to load texture");
        return(1);
    }

    dx_device->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
    dx_device->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
      
	D3DXMatrixPerspectiveFovLH(&dx_projection,  D3DX_PI * 0.29f,     // FOV
                    		   (float)display_x / (float)display_y, // Aspect ratio
                    		   1.0f, 1000.0f);

    terrain.setup();
                                     
    return(0);
}
/* End of graphics_start */



/* Free objects for graphics system */
void world_class::stop_graphics(void)
{
    if (dx_texture1) dx_texture1->Release();
    if (dx_texture2) dx_texture2->Release();

    if (terrain_buff) terrain_buff->Release();

    if (dx_device) dx_device->Release();    
    if (dx_object) dx_object->Release(); 
}
/* End of graphics_stop */






