/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: main.cpp
*/

#include "core.h"


bool running = true; // True if program running 


/* Windows callback */
LRESULT CALLBACK win_proc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
    switch(msg)
    {
        case WM_CREATE:
              SetCursorPos(200,200);
              SetTimer(hwnd, ID_BASE, 250, NULL);
            return(0);

        case WM_TIMER:
            world.update_fps();
            return(0);

        case WM_KEYDOWN:
        {


            return(0);
        }

        case WM_CHAR:
        {
            if (w == 27) DestroyWindow(hwnd);   // Escape exits

            
            if (w == 'v') world.toggle_wireframe();
            if (w == 'b') world.toggle_drawinternal();

            if (w == 'p') world.toggle_paused();
            if (w == 'l') world.toggle_slowmode();

            if (w == 'z') world.toggle_gravity();
            if (w == 'c') world.toggle_snap();
      
            return(0);
        }
               
               
        case WM_MOUSEWHEEL: // Mouse whhel controls vert tilt
        {
            int pos = (int)(HIWORD(w));

            // Add delta to camera velocity in the Y direction                       
            world.update_camera_vel(D3DXVECTOR3(0.0,0.001 * (double)(GET_WHEEL_DELTA_WPARAM(w)),0.0));

            return(0);
        }
                                         
        case WM_RBUTTONDOWN:
        {
          //  world.add_object();
            return(0);
        }
                 
        case WM_LBUTTONDOWN:
        {
         //   world.add_projectile();
            return(0);
        }            
        
        case WM_DESTROY:
        {
            PostQuitMessage(0);
            running = false;
            return 0;
        } 
    }

    return DefWindowProc (hwnd, msg, w, l);
}
/* End of win_proc */


/* Windows entry point */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    HWND hWnd;
    WNDCLASSEX wc;

    ZeroMemory(&wc, sizeof(WNDCLASSEX));

    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW |CS_OWNDC;
    wc.lpfnWndProc = win_proc;
    wc.hInstance = hInstance;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    
    wc.lpszClassName = "Quad";

    RegisterClassEx(&wc);

	HWND hwnd = CreateWindow("Quad", "Quadtree Terrain - 2007 Jason Hunt" , WS_EX_TOPMOST,
                        0, 0, world.get_display_x(), world.get_display_y(),
                		0, 0, hInstance, 0); 

    if (hwnd == NULL) return(1);


    world.start(hwnd);
    
  
    ShowCursor(FALSE);

    ShowWindow(hwnd, nCmdShow);

    MSG msg;
    
    int last_time = GetTickCount();
    
    // Modified windows message loop
    while(running)
    {
        if (PeekMessage(&msg, hwnd, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }


        int cur_time = GetTickCount();
        
        if (cur_time - last_time > 20)
        {
            last_time = cur_time;
            world.next_frame();
        }
    }

    ShowCursor(TRUE);

    world.stop();

    return msg.wParam;
}
/* End of WinMain */


