
#include "core.h"

//double camera_ang = 0;

double dist_res = 4.5;

unsigned int bound_total = 0;

unsigned int max_levels = 8;       // Maximun number of tree levels
                                   // it will try to subdivide
                                
    
bool debug_tree = false;            // True if we will dump tree on load



terrain_class::terrain_class( void )
{
    main_tree = NULL;         // Main tree for testing
}



void terrain_class::render( void )
{
     debug("terrain_class::render");

    // Reset number or triangles in pass and clear triangle list

    num_triangles = 0;

    t_list = NULL;
    
    main_tree->render();

   
     // Get vertex buffer
    if (world.dx_device->CreateVertexBuffer(num_triangles * 3 * sizeof(vertex),
                                            0, dx_fvf,
                                            D3DPOOL_MANAGED,
                                            &world.terrain_buff, NULL) != D3D_OK) return;
    VOID *data; 

    // Try to lock the vertex buffer
    if (world.terrain_buff ->Lock(0, 0, (void**)&data, 0) != D3D_OK) return;
    

    vertex *tmp = new vertex [num_triangles*3];


    unsigned int index = 0;

    triangle *t = t_list;

    while(t)
    {
        tmp[index++] = t->v1;
        tmp[index++] = t->v2;
        tmp[index++] = t->v3;
                
        t=t->next;
    }


    memcpy(data, tmp, sizeof(vertex) * num_triangles  * 3);

    world.terrain_buff->Unlock();

    // Set active texture
    world.dx_device->SetTexture(0,world.dx_texture2);

    // set terrain as source
    world.dx_device->SetStreamSource(0, world.terrain_buff, 0, sizeof(vertex));

    // Draw the primitives
    if (world.dx_device->DrawPrimitive(D3DPT_TRIANGLELIST, 0, num_triangles) != D3D_OK) return;  
    
    if (world.terrain_buff) world.terrain_buff->Release();    

    
    delete [] tmp;
}





bool quadtree::render( void )
{
    //  debug(" quadtree::render");

    // See if bounding box is visible

    // Get corner points
    point2d p1(min_x, min_y);
    point2d p2(max_x, min_y);
    point2d p3(max_x, max_y);
    point2d p4(min_x, max_y);
    
    
    point2d pos;
    
    pos.x = world.camera_pos.x;
    pos.y = world.camera_pos.z;
  
    
       
    // Entirly to the left of the left frustum line
/*    if (cross (world.camera_l_bound, p1 - pos) < 0 &&
        cross (world.camera_l_bound, p2 - pos) < 0 &&
        cross (world.camera_l_bound, p3 - pos) < 0 &&
        cross (world.camera_l_bound, p4 - pos) < 0 ) return(true);
        
        
    // Entirly to the right of the right frustum line
    if (cross (world.camera_r_bound, p1 - pos) > 0 &&
        cross (world.camera_r_bound, p2 - pos) > 0 &&
        cross (world.camera_r_bound, p3 - pos) > 0 &&
        cross (world.camera_r_bound, p4 - pos) > 0 ) return(true);

*/
    // Otherwise, draw it

    
    bool child_render = false;
    

    double diff_x = pos.x - (min_x + (max_x - min_x) / 2.0);
    double diff_y = pos.y - (min_y + (max_y - min_y) / 2.0);
  
    double dist = diff_x * diff_x + diff_y * diff_y;


    double x_size = world.world_max.x - world.world_min.x;
    double y_size = world.world_max.y - world.world_min.y;
    
    double cut = (x_size * x_size + y_size * y_size) / pow(dist_res, max_levels - level );

    
    
    // If it's not a leaf is must have children
    if (!is_leaf && dist < cut )
    {
    
    //if (!is_leaf)
   // {
        if (nodes[0]->render()) child_render = true;
        if (nodes[1]->render()) child_render = true;
        if (nodes[2]->render()) child_render = true;
        if (nodes[3]->render()) child_render = true;
    }


    if (child_render) return(true);    
    
    
    
    //    add the two triangles to the triangle list
    
    world.terrain.num_triangles += 2;
    
    triangle *tmp = new triangle;
    
    tmp->next = world.terrain.t_list;
    
    
    world.terrain.t_list = tmp;
    

//    tmp->v1 = vertex(min_x, min_y, 0.0);
//    tmp->v2 = vertex(max_x, min_y, 0.0);
//    tmp->v3 = vertex(max_x, max_y, 0.0);
  
  
    double tx1 = min_x / (world.world_max.x - world.world_min.x);
    double tx2 = max_x / (world.world_max.x - world.world_min.x);

    double ty1 = min_y / (world.world_max.y - world.world_min.y);
    double ty2 = max_y / (world.world_max.y - world.world_min.y);
   
    
    tmp->v1 = vertex(vector(min_x, 0, min_y), tx1, ty1);  
    tmp->v2 = vertex(vector(max_x, 0, max_y), tx2, ty2);  
    tmp->v3 = vertex(vector(max_x, 0, min_y), tx2, ty1);  

        
    tmp = new triangle;
    
    tmp->next = world.terrain.t_list;
    
    
    world.terrain.t_list = tmp;   
    
    tmp->v1 = vertex(vector(min_x, 0, min_y), tx1, ty1);  
    tmp->v2 = vertex(vector(min_x, 0, max_y), tx1, ty2);  
    tmp->v3 = vertex(vector(max_x, 0, max_y), tx2, ty2);  


  
    // render this quad
    
    
//    int x1 = screen_x(min_x);
//    int y1 = screen_y(min_y);
//    int x2 = screen_x(max_x);
//    int y2 = screen_y(max_y);

//    MoveToEx(memdc, x1, y1, NULL);
//    LineTo(memdc, x2, y1);
//    LineTo(memdc, x2, y2);
//    LineTo(memdc, x1, y2);
//    LineTo(memdc, x1, y1);


    return(true);
}


/* Setup a test quadtree */
void terrain_class::setup( void )
{
    debug("terrain_class::setup");


    // Create terrain quadtree 
    
    
    
    main_tree = new quadtree(world.world_min.x, world.world_min.z, 
                             world.world_max.x, world.world_max.z, max_levels);

//    if (debug_tree)
//        main_tree->dump(0);   // Dump main tree for debugging
}
/* End of setup quadtree */




/* Creates a quadtree from a list of triangles and a bounding dimension */
quadtree::quadtree(double nmin_x, double nmin_y,  double nmax_x, double nmax_y, int l)
{
    min_x = nmin_x;   max_x = nmax_x;
    min_y = nmin_y;   max_y = nmax_y;    

    is_leaf = true;      // Default to a leaf with no nodes

    bound_total += 1;

    if (l <= 0) return; // Too many levels

    level = l;

    // Compute center of bounding region
    double mid_x = min_x + (max_x - min_x) / 2.0;
    double mid_y = min_y + (max_y - min_y) / 2.0;

    nodes[0] = new quadtree(min_x, min_y, mid_x, mid_y, l-1);
    nodes[1] = new quadtree(mid_x, min_y, max_x, mid_y, l-1);
    nodes[2] = new quadtree(min_x, mid_y, mid_x, max_y, l-1);
    nodes[3] = new quadtree(mid_x, mid_y, max_x, max_y, l-1);    

    is_leaf = false;    // Node is not a leaf
}
/* End of quadtree constructor */



quadtree::~quadtree()
{    
    if (!is_leaf)
    {
        delete nodes[0];
        delete nodes[1];
        delete nodes[2];
        delete nodes[3];
    }
    
}


