/*
    Physics Simulation

    (C) 2007 Jason Hunt
    nulluser@gmail.com

    File: world.h
*/


/* Definition of world class */
class world_class 
{

    friend class terrain_class;
    friend class quadtree;

    public:
    world_class();
    
    int start(HWND hwnd);    
    void stop(void);    
      
    void create_terrain( void );
  
    void  render_terrain(void);
  
    void update_camera( void );
    void display_text(int x, int y, char *text);
    void showstat( void );    
    void render(void);
    int start_graphics(HWND hwnd);
    void stop_graphics(void);
    
    void update_fps( void );    
        
    void next_frame( void );

//    void free_objects(void);
//    void add_object(void);
//    void add_projectile(void);

  //  void check_object_collide(dobject_class *obj1, dobject_class *obj2);          
   // void check_collide(void);
    
    unsigned int get_display_x(void){ return display_x; };
    unsigned int get_display_y(void){ return display_y; };
    
    
    void update_camera_vel(D3DXVECTOR3 v) { camera_vel += v; };
    
    
    void toggle_wireframe( void ) {wireframe = !wireframe; };
    void toggle_drawinternal( void ) { drawinternal = ! drawinternal; };



    void toggle_paused( void ) { paused = !paused; };
    void toggle_slowmode( void ) { slowmode = !slowmode; };

    void toggle_gravity( void ) { apply_grav = !apply_grav; };
    void toggle_snap( void ) { apply_snap= ! apply_snap; };
      
    double get_delta_t( void ) { return(slowmode ? rk4_delta_slow : rk4_delta_fast);};


    //dobject_class * get_objects( void ) { return objects; };

private:

    // Direct X stuff
    
     D3DXVECTOR3 probe_pos;                 // Position of probe in front of camera
     double probe_dist;                   // Distance from camrea to probe
     double probe_radius;                   // Side of probe
    
    D3DXVECTOR3 camera_pos;
    D3DXVECTOR3 camera_vel;
    D3DXVECTOR3 camera_dir;                 // Direction of camera

    vector camera_ang;                      // Current angle of camera

    point2d camera_l_bound;
    point2d camera_r_bound;



    LPDIRECT3D9 dx_object;                  // Direct3d Object
    LPDIRECT3DDEVICE9 dx_device;            // Actual device

    LPD3DXFONT dx_font;                     // Test font
    LPDIRECT3DTEXTURE9 dx_texture1;         // Test texture
    LPDIRECT3DTEXTURE9 dx_texture2;         // Test texture

    D3DCOLOR dx_white, dx_black;            // Colors for text

  	D3DXMATRIX dx_projection;               // Projection transform
    D3DXMATRIX dx_view;                     // View transform

    LPDIRECT3DVERTEXBUFFER9 terrain_buff;   // Vertex Buffer

  
    unsigned int display_x, display_y;      // Screen size
    unsigned int frames;                    // Number of frames in this 250ms
    unsigned int fps;                       // Frames per second
  
    // Toggles  
    bool paused;                            // True if simulation is stopped
    bool apply_grav;                        // True if gravity is enabled
    bool apply_snap;                        // True if breaking is enabled
    bool slowmode;                          // True if simulation is in slow mode
    bool fullscreen;                        // True for fullscreen
    bool wireframe;                         // True to draw wireframe
    bool drawinternal;                      // True to draw internal structure

  
    vector world_min, world_max;    // World size in meters

    int terrain_x_cells, terrain_z_cells;    // Terrain cells

    unsigned int terrain_triangles;

    // Parameters

    double cam_speed;                       // Camera Speed

    double grav;                            // Gravity in meters / second
    double rk4_delta_fast;                  // Delta time for RK4 integration
    double rk4_delta_slow;                  // Delta time for RK4 integration
   
    double terrain_force;
    double camera_force;
    double object_force;
  
   // dobject_class *objects;                 // Linked list of all dynamic objects
    int num_dobjects;


    terrain_class terrain;

};

extern world_class world;

